package de.ozgcloud.vorgang.vorgang;

import java.util.Optional;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

@Mapper
interface VorgangStubMapper {

	@Mapping(target = "deletedAt", ignore = true)
	@Mapping(target = "status", constant = "DELETED")
	@Mapping(target = "header", expression = "java(extractStubHeader(vorgang))")
	@Mapping(target = "eingangs", expression = "java(extractStubEingang(vorgang))")
	VorgangStub fromVorgang(Vorgang vorgang);

	default VorgangHeadStub extractStubHeader(Vorgang vorgang) {
		var builder = VorgangHeadStub.builder();
		getOrganisationsEinheitenId(vorgang).ifPresent(builder::organisationsEinheitId);
		return builder.build();
	}

	default EingangStub extractStubEingang(Vorgang vorgang) {
		var builder = EingangStub.builder();
		getOrganisationsEinheitenId(vorgang)
				.map(orgaId -> ZustaendigeStelleStub.builder().organisationseinheitenId(orgaId).build())
				.ifPresent(builder::zustaendigeStelle);
		return builder.build();

	}

	default Optional<String> getOrganisationsEinheitenId(Vorgang vorgang) {
		try {
			return Optional.ofNullable(vorgang.getEingangs().get(0).getZustaendigeStelle().getOrganisationseinheitenId());
		} catch (NullPointerException | IndexOutOfBoundsException e) {
			return Optional.empty();
		}
	}

}
