/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandCreatedEvent;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.vorgang.VorgangManagerServerApplication;
import de.ozgcloud.vorgang.command.CommandService;
import de.ozgcloud.vorgang.command.CommandTestFactory;
import de.ozgcloud.vorgang.vorgang.VorgangService;

@ITCase
@SpringBootTest(classes = { VorgangManagerServerApplication.class })
class PostfachEventListenerITCase {

	@Autowired
	private ApplicationEventPublisher publisher;

	@Mock
	private Command command;

	@SpyBean
	private PostfachEventListener listener;

	@MockBean
	private VorgangService vorgangService;

	@MockBean
	private CommandService commandService;

	@Nested
	class TestSendPostfachNachricht {

		@BeforeEach
		void mockCommand() {
			when(command.getOrder()).thenReturn("SEND_POSTFACH_NACHRICHT");
		}

		@BeforeEach
		void initServices() {
			when(commandService.findById(any())).thenReturn(Optional.of(CommandTestFactory.create()));
		}

		@Test
		void shouldCallService() {
			doNothing().when(listener).sendPostfachNachricht(any());
			var event = new CommandCreatedEvent(command);

			publisher.publishEvent(event);

			verify(listener, timeout(500)).sendPostfachNachricht(any());
		}
	}

}
